@@ -24,7 +24,7 @@ def turn_to_leaf(nid, start_idx, n_samples, idx, values_idx_array, values_si_idx
 
 @jit
 def bfs_loop(queue_size, n_nodes, max_features, new_idx_array, idx_array, new_si_idx_array, new_nid_array, left_children, right_children,
-    feature_idx_array, feature_threshold_array, nid_array, imp_min, min_split, feature_idx, si_idx_array, subset_indices_array, threshold, min_samples_split,
+    feature_idx_array, feature_threshold_array, nid_array, imp_min, min_split, feature_idx, si_idx_array, threshold, min_samples_split,
     values_idx_array, values_si_idx_array):
   new_queue_size = 0
 
@@ -81,8 +81,7 @@ def bfs_loop(queue_size, n_nodes, max_features, new_idx_array, idx_array, new_si
           new_nid_array[new_queue_size] = right_nid
           new_queue_size += 1
       else:
-        turn_to_leaf(right_nid, col + 1, 1, si_idx, values_idx_array, values_si_idx_array)
-   
+        turn_to_leaf(right_nid, col + 1, 1, si_idx, values_idx_array, values_si_idx_array)   
   return n_nodes , new_queue_size, new_idx_array, new_si_idx_array, new_nid_array
 
 
@@ -112,6 +111,9 @@ class RandomDecisionTreeSmall(RandomBaseTree):
   
   def get_indices(self):
     return np.array(random.sample(xrange(self.n_features), self.max_features), dtype=self.dtype_indices)
+  
+  def __shuffle_features(self):
+    np.random.shuffle(self.features_array)
 
   def __compile_kernels(self):
     ctype_indices = dtype_to_ctype(self.dtype_indices)
@@ -121,67 +123,112 @@ class RandomDecisionTreeSmall(RandomBaseTree):
     n_labels = self.n_labels
     n_threads = self.COMPT_THREADS_PER_BLOCK
     n_shf_threads = self.RESHUFFLE_THREADS_PER_BLOCK
-    
-    self.fill_kernel = mk_kernel((ctype_indices,), "fill_table", "fill_table_si.cu") 
-    self.scan_total_kernel = mk_kernel((n_threads, n_labels, ctype_labels, ctype_counts, ctype_indices), 
-        "count_total", "scan_kernel_total_si.cu") 
-    
-    self.comput_total_kernel = mk_kernel((n_threads, n_labels, ctype_samples, ctype_labels, 
-      ctype_counts, ctype_indices), "compute", "comput_kernel_total_rand.cu")
-     
-    self.scan_reshuffle_tex, tex_ref = mk_tex_kernel((ctype_indices, n_shf_threads), 
-        "scan_reshuffle", "tex_mark", "pos_scan_reshuffle_si_c_tex.cu")   
-    self.mark_table.bind_to_texref_ext(tex_ref) 
-    
-    self.find_min_kernel = mk_kernel((ctype_counts, 32), 
-        "find_min_imp", "find_min_gini.cu")
-      
-    self.predict_kernel = mk_kernel((ctype_indices, ctype_samples, ctype_labels), 
-        "predict", "predict.cu")
-  
-    self.scan_total_bfs = mk_kernel((self.BFS_THREADS, n_labels, ctype_labels, ctype_counts, ctype_indices), 
-        "count_total", "scan_kernel_total_bfs.cu")
-  
-    self.comput_bfs = mk_kernel((self.BFS_THREADS, n_labels, ctype_samples, ctype_labels, ctype_counts, 
-      ctype_indices), "compute", "comput_kernel_bfs.cu")
-    
-    self.fill_bfs = mk_kernel((ctype_indices,), "fill_table", "fill_table_bfs.cu")
-    
-    self.reshuffle_bfs, tex_ref = mk_tex_kernel((ctype_indices, self.BFS_THREADS), 
-        "scan_reshuffle", "tex_mark", "pos_scan_reshuffle_bfs.cu")
-    self.mark_table.bind_to_texref_ext(tex_ref) 
-    
-    self.comput_total_2d = mk_kernel((n_threads, n_labels, ctype_samples, ctype_labels, ctype_counts, ctype_indices, 
-      self.MAX_BLOCK_PER_FEATURE), "compute", "comput_kernel_2d.cu")
 
-    self.reduce_2d = mk_kernel((ctype_indices, self.MAX_BLOCK_PER_FEATURE), "reduce", "reduce_2d.cu")
+    self.fill_kernel = mk_kernel(
+      params = (ctype_indices,), 
+      func_name = "fill_table", 
+      kernel_file = "fill_table_si.cu", 
+      prepare_args = "PiiPi")
+
+      
+    self.scan_total_kernel = mk_kernel(
+        params = (n_threads, n_labels, ctype_labels, ctype_counts, ctype_indices), 
+        func_name = "count_total", 
+        kernel_file = "scan_kernel_total_si.cu", 
+        prepare_args = "PPPi") 
     
-    self.scan_total_2d = mk_kernel((n_threads, n_labels, ctype_labels, ctype_counts, ctype_indices, self.MAX_BLOCK_PER_FEATURE),
-        "count_total", "scan_kernel_2d.cu")
+    self.comput_total_kernel = mk_kernel(
+      params = (n_threads, n_labels, ctype_samples, 
+                ctype_labels, ctype_counts, ctype_indices), 
+      func_name = "compute", 
+      kernel_file = "comput_kernel_total_rand.cu", 
+      prepare_args = "PPPPPPPPii")
+         
+    self.scan_reshuffle_tex, tex_ref = mk_tex_kernel(
+      params = (ctype_indices, n_shf_threads), 
+      func_name = "scan_reshuffle", 
+      tex_name = "tex_mark", 
+      kernel_file = "pos_scan_reshuffle_si_c_tex.cu", 
+      prepare_args = "PPPiii")   
+    self.mark_table.bind_to_texref_ext(tex_ref) 
     
-    self.scan_reduce = mk_kernel((n_labels, ctype_indices, self.MAX_BLOCK_PER_FEATURE), "scan_reduce", "scan_reduce.cu")
     
-    self.get_thresholds = mk_kernel((ctype_indices, ctype_samples), "get_thresholds", "get_thresholds.cu")
+    self.find_min_kernel = mk_kernel(
+      params = (ctype_counts, 32), 
+      func_name = "find_min_imp", 
+      kernel_file = "find_min_gini.cu", 
+      prepare_args = "PPPi")
     
-    if hasattr(self.fill_kernel, "is_prepared"):
-      return
+    self.predict_kernel = mk_kernel(
+        params = (ctype_indices, ctype_samples, ctype_labels), 
+        func_name = "predict", 
+        kernel_file = "predict.cu", 
+        prepare_args = "PPPPPPPii")
+  
+    self.scan_total_bfs = mk_kernel(
+      params = (self.BFS_THREADS, n_labels, ctype_labels, ctype_counts, ctype_indices), 
+      func_name = "count_total", 
+      kernel_file = "scan_kernel_total_bfs.cu", 
+      prepare_args = "PPPPPP")
+  
+    self.comput_bfs = mk_kernel(
+      params = (self.BFS_THREADS, n_labels, ctype_samples, 
+                ctype_labels, ctype_counts, ctype_indices), 
+      func_name = "compute", 
+      kernel_file = "comput_kernel_bfs.cu", 
+      prepare_args = "PPPPPPPPPPPiii")
     
-    self.fill_kernel.is_prepared = True
-    self.fill_kernel.prepare("PiiPi")
-    self.scan_reshuffle_tex.prepare("PPPiii") 
-    self.scan_total_kernel.prepare("PPPi")
-    self.find_min_kernel.prepare("PPPi")
-    self.predict_kernel.prepare("PPPPPPPii")
-    self.scan_total_bfs.prepare("PPPPPPPi")
-    self.comput_bfs.prepare("PPPPPPPPPPPii")
-    self.fill_bfs.prepare("PPPPPPPi")
-    self.reshuffle_bfs.prepare("PPPPPPii")
-    self.comput_total_kernel.prepare("PPPPPPPPii")
-    self.comput_total_2d.prepare("PPPPPPPiii")
-    self.reduce_2d.prepare("PPPPPi")
-    self.scan_total_2d.prepare("PPPPiii")
-    self.scan_reduce.prepare("Pi")
-    self.get_thresholds.prepare("PPPPPPPi")
+    self.fill_bfs = mk_kernel(
+      params = (ctype_indices,), 
+      func_name = "fill_table", 
+      kernel_file = "fill_table_bfs.cu", 
+      prepare_args = "PPPPPPPi")
+    
+    self.reshuffle_bfs, tex_ref = mk_tex_kernel(
+      params = (ctype_indices, self.BFS_THREADS), 
+      func_name = "scan_reshuffle", 
+      tex_name= "tex_mark", 
+      kernel_file = "pos_scan_reshuffle_bfs.cu", 
+      prepare_args = "PPPPPPii")
+    self.mark_table.bind_to_texref_ext(tex_ref) 
+    
+    self.comput_total_2d = mk_kernel(
+      params = (n_threads, n_labels, ctype_samples, ctype_labels, ctype_counts, 
+                ctype_indices, self.MAX_BLOCK_PER_FEATURE), 
+      func_name = "compute", 
+      kernel_file = "comput_kernel_2d.cu", 
+      prepare_args = "PPPPPPPiii")
+
+    self.reduce_2d = mk_kernel(
+      params = (ctype_indices, self.MAX_BLOCK_PER_FEATURE), 
+      func_name = "reduce", 
+      kernel_file = "reduce_2d.cu", 
+      prepare_args = "PPPPPi")
+    
+    self.scan_total_2d = mk_kernel(
+      params = (n_threads, n_labels, ctype_labels, ctype_counts, 
+                ctype_indices, self.MAX_BLOCK_PER_FEATURE),
+      func_name = "count_total", 
+      kernel_file = "scan_kernel_2d.cu", 
+      prepare_args = "PPPPiii")
+    
+    self.scan_reduce = mk_kernel(
+      params = (n_labels, ctype_indices, self.MAX_BLOCK_PER_FEATURE), 
+      func_name = "scan_reduce", 
+      kernel_file = "scan_reduce.cu", 
+      prepare_args = "Pi")
+    
+    self.get_thresholds = mk_kernel(
+      params = (ctype_indices, ctype_samples), 
+      func_name = "get_thresholds", 
+      kernel_file = "get_thresholds.cu", 
+      prepare_args = "PPPPPPPi")
+    
+    self.feature_selector = mk_kernel(
+      params = (ctype_indices, ctype_samples), 
+      func_name = "feature_selector", 
+      kernel_file = "feature_selector.cu", 
+      prepare_args = "PPPii")
 
   def __allocate_gpuarrays(self):
     if self.max_features < 4:
@@ -197,6 +244,7 @@ class RandomDecisionTreeSmall(RandomBaseTree):
     self.label_total_2d = gpuarray.zeros(self.max_features * (self.MAX_BLOCK_PER_FEATURE + 1) * self.n_labels, self.dtype_indices)
     self.impurity_2d = gpuarray.empty(self.max_features * self.MAX_BLOCK_PER_FEATURE * 2, np.float32)
     self.min_split_2d = gpuarray.empty(self.max_features * self.MAX_BLOCK_PER_FEATURE, self.dtype_counts)
+    self.feature_mask = gpuarray.empty(self.n_features, np.uint8)
 
   def __release_gpuarrays(self):
     self.impurity_left = None
@@ -219,6 +267,29 @@ class RandomDecisionTreeSmall(RandomBaseTree):
     self.label_total_2d = None
     self.min_split_2d = None
     self.impurity_2d = None
+    self.feature_mask = None
+  
+  def __allocate_numpyarrays(self):
+    self.left_children = np.zeros(self.n_samples * 2, dtype = np.uint32)
+    self.right_children = np.zeros(self.n_samples * 2, dtype = np.uint32) 
+    self.feature_idx_array = np.zeros(2 * self.n_samples, dtype = np.uint16)
+    self.feature_threshold_array = np.zeros(2 * self.n_samples, dtype = np.float32)
+    self.idx_array = np.zeros(2 * self.n_samples, dtype = np.uint32)
+    self.si_idx_array = np.zeros(self.n_samples, dtype = np.uint8)
+    self.nid_array = np.zeros(self.n_samples, dtype = np.uint32)
+    self.values_idx_array = np.zeros(2 * self.n_samples, dtype = self.dtype_indices)
+    self.values_si_idx_array = np.zeros(2 * self.n_samples, dtype = np.uint8)
+    self.features_array = np.arange(self.n_features, dtype = self.dtype_indices)
+    self.threshold_value_idx = np.zeros(2, self.dtype_indices)
+    self.min_imp_info = np.zeros(4, dtype = np.float32)  
+
+  def __release_numpyarrays(self):
+    self.feature_array = None
+    self.nid_array = None
+    self.idx_array = None
+    self.si_idx_array = None
+    self.threshold_value_idx = None
+    self.min_imp_info = None
 
   def __bfs_construct(self):
     while self.queue_size > 0:
@@ -227,7 +298,7 @@ class RandomDecisionTreeSmall(RandomBaseTree):
   def __bfs(self):
     idx_array_gpu = gpuarray.to_gpu(self.idx_array[0 : self.queue_size * 2])
     si_idx_array_gpu = gpuarray.to_gpu(self.si_idx_array[0 : self.queue_size])
-    subset_indices_array_gpu = gpuarray.empty(self.queue_size * self.max_features, dtype = self.dtype_indices)
+    subset_indices_array_gpu = gpuarray.empty(self.n_features, dtype = self.dtype_indices)
     min_feature_idx_gpu = gpuarray.empty(self.queue_size, dtype = np.uint16)
     
     self.label_total = gpuarray.empty(self.queue_size * self.n_labels, dtype = self.dtype_counts)  
@@ -235,7 +306,7 @@ class RandomDecisionTreeSmall(RandomBaseTree):
     self.min_split = gpuarray.empty(self.queue_size, dtype = self.dtype_indices) 
     threshold_value = gpuarray.empty(self.queue_size, dtype = np.float32)
 
-    cuda.memcpy_htod(subset_indices_array_gpu.ptr, self.subset_indices_array[0:self.max_features * self.queue_size]) 
+    cuda.memcpy_htod(subset_indices_array_gpu.ptr, self.features_array) 
     
     self.scan_total_bfs.prepared_call(
             (self.queue_size, 1),
@@ -245,9 +316,7 @@ class RandomDecisionTreeSmall(RandomBaseTree):
             self.labels_gpu.ptr,
             self.label_total.ptr,
             si_idx_array_gpu.ptr,
-            idx_array_gpu.ptr,
-            subset_indices_array_gpu.ptr,
-            self.max_features)
+            idx_array_gpu.ptr)
     
     self.comput_bfs.prepared_call(
           (self.queue_size, 1),
@@ -264,6 +333,7 @@ class RandomDecisionTreeSmall(RandomBaseTree):
           self.min_split.ptr,
           min_feature_idx_gpu.ptr,
           self.max_features,
+          self.n_features,
           self.stride)
         
     self.fill_bfs.prepared_call(
@@ -278,7 +348,6 @@ class RandomDecisionTreeSmall(RandomBaseTree):
           self.mark_table.ptr,
           self.stride)
 
-
     block_per_split = int(math.ceil(float(2000) / self.queue_size))
     if block_per_split > self.n_features:
       block_per_split = self.n_features
@@ -295,10 +364,9 @@ class RandomDecisionTreeSmall(RandomBaseTree):
           self.n_features,
           self.stride)
     
-    max_features = self.max_features
-    old_queue_size = self.queue_size
-    for i in range(old_queue_size):
-      self.subset_indices_array[i * max_features : (i + 1) * max_features] = self.get_indices() 
+
+    
+    self.__shuffle_features()
 
     self.get_thresholds.prepared_call(
           (self.queue_size, 1),
@@ -312,8 +380,7 @@ class RandomDecisionTreeSmall(RandomBaseTree):
           self.min_split.ptr,
           self.stride)
     
-    queue_size = 0
-    n_nodes = self.n_nodes
+
     new_idx_array = np.empty(self.queue_size * 2 * 2, dtype = np.uint32)
     idx_array = self.idx_array
     new_si_idx_array = np.empty(self.queue_size * 2, dtype = np.uint8)
@@ -331,13 +398,9 @@ class RandomDecisionTreeSmall(RandomBaseTree):
     
     self.n_nodes, self.queue_size, self.idx_array, self.si_idx_array, self.nid_array = bfs_loop(self.queue_size, self.n_nodes, 
         self.max_features, new_idx_array, idx_array, new_si_idx_array, new_nid_array, left_children, right_children,
-        feature_idx_array, feature_threshold_array, nid_array, imp_min, min_split, feature_idx, si_idx_array, self.subset_indices_array, threshold,
+        feature_idx_array, feature_threshold_array, nid_array, imp_min, min_split, feature_idx, si_idx_array, threshold,
         self.min_samples_split, self.values_idx_array, self.values_si_idx_array)
 
-    if self.queue_size > old_queue_size:
-      for i in range(old_queue_size, self.queue_size):
-        self.subset_indices_array[i * max_features : (i + 1) * max_features] = self.get_indices() 
-
     self.n_nodes = int(self.n_nodes)
     self.queue_size = int(self.queue_size)
  
@@ -345,11 +408,10 @@ class RandomDecisionTreeSmall(RandomBaseTree):
   def fit(self, samples, target, sorted_indices, n_samples): 
     self.samples_itemsize = self.dtype_samples.itemsize
     self.labels_itemsize = self.dtype_labels.itemsize
-    self.threshold_value_idx = np.zeros(2, self.dtype_indices)
-    self.min_imp_info = np.zeros(4, dtype = np.float32)  
     
     self.__allocate_gpuarrays()
     self.__compile_kernels() 
+    
     self.sorted_indices_gpu = sorted_indices 
     self.sorted_indices_gpu_ = self.sorted_indices_gpu.copy()
     self.n_samples = n_samples    
@@ -362,24 +424,17 @@ class RandomDecisionTreeSmall(RandomBaseTree):
     
     self.samples = samples
     self.target = target
-    self.left_children = np.zeros(self.n_samples * 2, dtype = np.uint32)
-    self.right_children = np.zeros(self.n_samples * 2, dtype = np.uint32)
-    
-    self.feature_idx_array = np.zeros(2 * self.n_samples, dtype = np.uint16)
-    self.feature_threshold_array = np.zeros(2 * self.n_samples, dtype = np.float32)
-    self.idx_array = np.zeros(2 * self.n_samples, dtype = np.uint32)
-    self.si_idx_array = np.zeros(self.n_samples, dtype = np.uint8)
-    self.subset_indices_array = np.zeros(self.n_samples * self.max_features, dtype = self.dtype_indices)
     self.queue_size = 0
-    self.nid_array = np.zeros(self.n_samples, dtype = np.uint32)
-    self.values_idx_array = np.zeros(2 * self.n_samples, dtype = self.dtype_indices)
-    self.values_si_idx_array = np.zeros(2 * self.n_samples, dtype = np.uint8)
 
+    self.__allocate_numpyarrays()
     self.n_nodes = 0 
-    self.root = self.__dfs_construct(1, 1.0, 0, self.n_samples, self.sorted_indices_gpu, self.sorted_indices_gpu_, self.get_indices())  
+
+    self.root = self.__dfs_construct(1, 1.0, 0, self.n_samples, self.sorted_indices_gpu, self.sorted_indices_gpu_)  
     self.__bfs_construct() 
     self.__gpu_decorate_nodes(samples, target)
     self.__release_gpuarrays() 
+    self.__release_numpyarrays()
+
 
   def __gpu_decorate_nodes(self, samples, labels):
     si_0 = np.empty(self.n_samples, dtype = self.dtype_indices)
@@ -407,7 +462,7 @@ class RandomDecisionTreeSmall(RandomBaseTree):
   def __gini_small(self, n_samples, indices_offset, subset_indices, si_gpu_in):
     block = (self.COMPT_THREADS_PER_BLOCK, 1, 1)
     grid = (self.max_features, 1) 
-    
+
     self.scan_total_kernel.prepared_call(
                 (1, 1),
                 block,
@@ -438,14 +493,12 @@ class RandomDecisionTreeSmall(RandomBaseTree):
                 self.min_split.ptr,
                 self.max_features)
     
-
     cuda.memcpy_dtoh(self.min_imp_info, self.impurity_left.ptr)
     min_right = self.min_imp_info[1] 
     min_left = self.min_imp_info[0] 
     col = int(self.min_imp_info[2]) 
     row = int(self.min_imp_info[3])
     row = subset_indices[row] 
-
     return min_left, min_right, row, col
 
 
@@ -458,11 +511,7 @@ class RandomDecisionTreeSmall(RandomBaseTree):
 
   def __gini_large(self, n_samples, indices_offset, subset_indices, si_gpu_in):
     n_block, n_range = self.__get_block_size(n_samples)
-    min_left = None
-    min_right = None
-    row = None
-    col = None
-    
+        
     self.scan_total_2d.prepared_call(
           (self.max_features, n_block),
           (self.COMPT_THREADS_PER_BLOCK, 1, 1),
@@ -521,7 +570,7 @@ class RandomDecisionTreeSmall(RandomBaseTree):
     return min_left, min_right, row, col
 
 
-  def  __dfs_construct(self, depth, error_rate, start_idx, stop_idx, si_gpu_in, si_gpu_out, subset_indices):
+  def  __dfs_construct(self, depth, error_rate, start_idx, stop_idx, si_gpu_in, si_gpu_out):
     def check_terminate():
       if error_rate == 0.0:
         return True
@@ -546,12 +595,34 @@ class RandomDecisionTreeSmall(RandomBaseTree):
       self.idx_array[self.queue_size * 2] = start_idx
       self.idx_array[self.queue_size * 2 + 1] = stop_idx
       self.si_idx_array[self.queue_size] = si_gpu_in.idx
-      self.subset_indices_array[self.queue_size * self.max_features : 
-          (self.queue_size + 1) * self.max_features] = subset_indices
       self.nid_array[self.queue_size] = nid
       self.queue_size += 1
       return
+     
+    self.feature_selector.prepared_call(
+              (self.n_features, 1),
+              (1, 1, 1),
+              si_gpu_in.ptr + indices_offset,
+              self.samples_gpu.ptr,
+              self.feature_mask.ptr,
+              n_samples,
+              self.stride)
     
+    selected_features = np.where(self.feature_mask.get())[0] 
+    feature_num = selected_features.size
+   
+    if feature_num == self.n_features:
+      subset_indices = self.get_indices()
+    else:
+      subset_indices = np.zeros(self.max_features, self.dtype_indices)
+      if feature_num < self.max_features:
+        max_features = feature_num
+      else:
+        max_features = self.max_features
+
+      indices = np.array(random.sample(xrange(feature_num), max_features))
+      subset_indices[0 : max_features] = selected_features[indices].astype(self.dtype_indices)
+
     cuda.memcpy_htod(self.subset_indices.ptr, subset_indices)
     
     if n_samples > 2000:
@@ -588,15 +659,11 @@ class RandomDecisionTreeSmall(RandomBaseTree):
                       n_samples,
                       col,
                       self.stride) 
-
-    
-    subset_indices_left = self.get_indices()
-    subset_indices_right = self.get_indices()
-    
+ 
     self.left_children[nid] = self.n_nodes
     self.__dfs_construct(depth + 1, min_left, 
-        start_idx, start_idx + col + 1, si_gpu_out, si_gpu_in, subset_indices_left)
+        start_idx, start_idx + col + 1, si_gpu_out, si_gpu_in)
     
     self.right_children[nid] = self.n_nodes
     self.__dfs_construct(depth + 1, min_right, 
-        start_idx + col + 1, stop_idx, si_gpu_out, si_gpu_in, subset_indices_right)
+        start_idx + col + 1, stop_idx, si_gpu_out, si_gpu_in)
