@@ -1,7 +1,7 @@
 import cPickle
 import numpy as np
 from os import path
-from sklearn.datasets import load_digits, load_iris
+from sklearn.datasets import load_digits, load_iris, load_diabetes, fetch_covtype 
 
 _img_data = None
 
@@ -16,6 +16,14 @@ def load_data(ds_name):
     ds = load_iris()
     x_train = ds.data
     y_train = ds.target
+  elif ds_name == "diabetes":
+    ds = load_diabetes()
+    x_train = ds.data 
+    y_train = ds.target > 140 
+  elif ds_name == "covtype":
+    ds = fetch_covtype(download_if_missing = True)
+    x_train = ds.data 
+    y_train = ds.target 
   elif ds_name == "db":
     with open(data_dir + "data_batch_1", "r") as f:
       ds = cPickle.load(f)
@@ -38,15 +46,16 @@ def load_data(ds_name):
       y_train = np.array(ds['fine_labels'])
   elif ds_name == "inet":
     if _img_data is None:
-      with open("/home/yl1912/imagenet-subset.pickle", "r") as f:
+      with open("/ssd/imagenet-subset.pickle", "r") as f:
         _img_data = cPickle.load(f)
     return _img_data['x'][0:10000],  _img_data['Y'][0:10000] 
   elif ds_name == "inet_test":
     if _img_data is None:
-      with open("~/imagenet-subset.pickle", "r") as f:
+      with open("/ssd/imagenet-subset.pickle", "r") as f:
         _img_data = cPickle.load(f)
     return _img_data['x'][10000:],  _img_data['Y'][10000:] 
-
+  else:
+    assert False, "Unrecognized data set name %s" % ds_name
   return x_train, y_train
 
 
